@@ -1,5 +1,5 @@
 /* comm -- compare two sorted files line by line.
-   Copyright (C) 1986, 1990, 1991, 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 86, 90, 91, 1995-2000 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,8 +23,16 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
+#include "closeout.h"
 #include "linebuffer.h"
 #include "error.h"
+#include "hard-locale.h"
+#include "memcoll.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "comm"
+
+#define AUTHORS "Richard Stallman and David MacKenzie"
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
@@ -33,6 +41,11 @@
 /* The name this program was run with. */
 char *program_name;
 
+#ifdef ENABLE_NLS
+/* Nonzero if the LC_COLLATE locale is hard.  */
+static int hard_LC_COLLATE;
+#endif
+
 /* If nonzero, print lines that are found only in file 1. */
 static int only_file_1;
 
@@ -42,22 +55,16 @@ static int only_file_2;
 /* If nonzero, print lines that are found in both files. */
 static int both;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output then exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {0, 0, 0, 0}
 };
 
 
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -78,7 +85,7 @@ Compare sorted files LEFT_FILE and RIGHT_FILE line by line.\n\
       --help      display this help and exit\n\
       --version   output version information and exit\n\
 "));
-      puts (_("\nReport bugs to textutils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -89,7 +96,7 @@ Compare sorted files LEFT_FILE and RIGHT_FILE line by line.\n\
    2 for a line only in file 2, 3 for a line in both. */
 
 static void
-writeline (struct linebuffer *line, FILE *stream, int class)
+writeline (const struct linebuffer *line, FILE *stream, int class)
 {
   switch (class)
     {
@@ -101,7 +108,7 @@ writeline (struct linebuffer *line, FILE *stream, int class)
     case 2:
       if (!only_file_2)
 	return;
-      /* Skip the tab stop for case 1, if we are printing case 1.  */
+      /* Print a TAB if we are printing lines from file 1.  */
       if (only_file_1)
 	putc ('\t', stream);
       break;
@@ -109,17 +116,16 @@ writeline (struct linebuffer *line, FILE *stream, int class)
     case 3:
       if (!both)
 	return;
-      /* Skip the tab stop for case 1, if we are printing case 1.  */
+      /* Print a TAB if we are printing lines from file 1.  */
       if (only_file_1)
 	putc ('\t', stream);
-      /* Skip the tab stop for case 2, if we are printing case 2.  */
+      /* Print a TAB if we are printing lines from file 2.  */
       if (only_file_2)
 	putc ('\t', stream);
       break;
     }
 
   fwrite (line->buffer, sizeof (char), line->length, stream);
-  putc ('\n', stream);
 }
 
 /* Compare INFILES[0] and INFILES[1].
@@ -148,8 +154,7 @@ compare_files (char **infiles)
     {
       initbuffer (&lb1[i]);
       thisline[i] = &lb1[i];
-      streams[i] = strcmp (infiles[i], "-")
-	? fopen (infiles[i], "r") : stdin;
+      streams[i] = (STREQ (infiles[i], "-") ? stdin : fopen (infiles[i], "r"));
       if (!streams[i])
 	{
 	  error (0, errno, "%s", infiles[i]);
@@ -171,11 +176,18 @@ compare_files (char **infiles)
 	order = -1;
       else
 	{
-	  /* Cannot use bcmp -- it only returns a boolean value. */
-	  order = memcmp (thisline[0]->buffer, thisline[1]->buffer,
-			  min (thisline[0]->length, thisline[1]->length));
-	  if (order == 0)
-	    order = thisline[0]->length - thisline[1]->length;
+#ifdef ENABLE_NLS
+	  if (hard_LC_COLLATE)
+	    order = memcoll (thisline[0]->buffer, thisline[0]->length - 1,
+			     thisline[1]->buffer, thisline[1]->length - 1);
+	  else
+#endif
+	    {
+	      size_t len = min (thisline[0]->length, thisline[1]->length) - 1;
+	      order = memcmp (thisline[0]->buffer, thisline[1]->buffer, len);
+	      if (order == 0)
+		order = thisline[0]->length - thisline[1]->length;
+	    }
 	}
 
       /* Output the line that is lesser. */
@@ -204,11 +216,6 @@ compare_files (char **infiles)
 	  ret = 1;
 	}
     }
-  if (ferror (stdout) || fclose (stdout) == EOF)
-    {
-      error (0, errno, _("write error"));
-      ret = 1;
-    }
   return ret;
 }
 
@@ -222,11 +229,17 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
+#ifdef ENABLE_NLS
+  hard_LC_COLLATE = hard_locale (LC_COLLATE);
+#endif
+
   only_file_1 = 1;
   only_file_2 = 1;
   both = 1;
 
-  while ((c = getopt_long (argc, argv, "123", long_options, (int *) 0)) != EOF)
+  while ((c = getopt_long (argc, argv, "123", long_options, NULL)) != -1)
     switch (c)
       {
       case 0:
@@ -244,19 +257,14 @@ main (int argc, char **argv)
 	both = 0;
 	break;
 
+      case_GETOPT_HELP_CHAR;
+
+      case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
       default:
 	usage (1);
       }
 
-  if (show_version)
-    {
-      printf ("comm (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (EXIT_SUCCESS);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (optind + 2 != argc)
     usage (1);
 
