@@ -1,5 +1,5 @@
 /* head -- output first part of file(s)
-   Copyright (C) 89, 90, 91, 1995-1999 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 1995-2000 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
+#include "closeout.h"
 #include "error.h"
 #include "xstrtol.h"
 #include "safe-read.h"
@@ -174,6 +175,9 @@ head_lines (const char *filename, int fd, uintmax_t lines_to_write)
 static int
 head (const char *filename, int fd, uintmax_t n_units, int count_lines)
 {
+  if (print_headers)
+    write_header (filename);
+
   if (count_lines)
     return head_lines (filename, fd, n_units);
   else
@@ -188,10 +192,7 @@ head_file (const char *filename, uintmax_t n_units, int count_lines)
   if (STREQ (filename, "-"))
     {
       have_read_stdin = 1;
-      filename = _("standard input");
-      if (print_headers)
-	write_header (filename);
-      return head (filename, 0, n_units, count_lines);
+      return head (_("standard input"), STDIN_FILENO, n_units, count_lines);
     }
   else
     {
@@ -200,8 +201,6 @@ head_file (const char *filename, uintmax_t n_units, int count_lines)
 	{
 	  int errors;
 
-	  if (print_headers)
-	    write_header (filename);
 	  errors = head (filename, fd, n_units, count_lines);
 	  if (close (fd) == 0)
 	    return errors;
@@ -211,7 +210,7 @@ head_file (const char *filename, uintmax_t n_units, int count_lines)
     }
 }
 
-/* Convert a string of digits, N_STRING, with a single, optional suffix
+/* Convert a string of decimal digits, N_STRING, with a single, optional suffix
    character (b, k, or m) to an integral value.  Upon successful conversion,
    return that value.  If it cannot be converted, give a diagnostic and exit.
    COUNT_LINES indicates whether N_STRING is a number of bytes or a number
@@ -223,7 +222,7 @@ string_to_integer (int count_lines, const char *n_string)
   strtol_error s_err;
   uintmax_t n;
 
-  s_err = xstrtoumax (n_string, NULL, 0, &n, "bkm");
+  s_err = xstrtoumax (n_string, NULL, 10, &n, "bkm");
 
   if (s_err == LONGINT_INVALID)
     {
@@ -263,6 +262,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   have_read_stdin = 0;
 
   print_headers = 0;
@@ -381,10 +382,8 @@ main (int argc, char **argv)
   for (; optind < argc; ++optind)
     exit_status |= head_file (argv[optind], n_units, count_lines);
 
-  if (have_read_stdin && close (0) < 0)
+  if (have_read_stdin && close (STDIN_FILENO) < 0)
     error (EXIT_FAILURE, errno, "-");
-  if (fclose (stdout) == EOF)
-    error (EXIT_FAILURE, errno, _("write error"));
 
   exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
