@@ -1,5 +1,5 @@
 /* od -- dump files in octal and other formats
-   Copyright (C) 92, 1995-2000 Free Software Foundation, Inc.
+   Copyright (C) 92, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -225,13 +225,15 @@ static FILE *in_stream;
 /* If nonzero, at least one of the files we read was standard input.  */
 static int have_read_stdin;
 
-#ifdef HAVE_UNSIGNED_LONG_LONG
-# define LONGEST_INTEGRAL_TYPE unsigned long long
+#if HAVE_UNSIGNED_LONG_LONG
+typedef unsigned long long ulonglong_t;
 #else
-# define LONGEST_INTEGRAL_TYPE long int
+/* This is just a place-holder to avoid a few `#if' directives.
+   In this case, the type isn't actually used.  */
+typedef unsigned long int ulonglong_t;
 #endif
 
-#define MAX_INTEGRAL_TYPE_SIZE sizeof(LONGEST_INTEGRAL_TYPE)
+#define MAX_INTEGRAL_TYPE_SIZE sizeof (ulonglong_t)
 static enum size_spec integral_type_size[MAX_INTEGRAL_TYPE_SIZE + 1];
 
 #define MAX_FP_TYPE_SIZE sizeof(LONG_DOUBLE)
@@ -435,19 +437,17 @@ print_long (off_t n_bytes, const char *block, const char *fmt_string)
     }
 }
 
-#ifdef HAVE_UNSIGNED_LONG_LONG
 static void
 print_long_long (off_t n_bytes, const char *block, const char *fmt_string)
 {
   off_t i;
-  for (i = n_bytes / sizeof (unsigned long long); i > 0; i--)
+  for (i = n_bytes / sizeof (ulonglong_t); i > 0; i--)
     {
-      unsigned long long tmp = *(const unsigned long long *) block;
+      ulonglong_t tmp = *(const ulonglong_t *) block;
       printf (fmt_string, tmp);
-      block += sizeof (unsigned long long);
+      block += sizeof (ulonglong_t);
     }
 }
-#endif
 
 static void
 print_float (off_t n_bytes, const char *block, const char *fmt_string)
@@ -1633,8 +1633,8 @@ main (int argc, char **argv)
   integral_type_size[sizeof (short int)] = SHORT;
   integral_type_size[sizeof (int)] = INT;
   integral_type_size[sizeof (long int)] = LONG;
-#ifdef HAVE_UNSIGNED_LONG_LONG
-  integral_type_size[sizeof (long long)] = LONG_LONG;
+#if HAVE_UNSIGNED_LONG_LONG
+  integral_type_size[sizeof (ulonglong_t)] = LONG_LONG;
 #endif
 
   for (i = 0; i <= MAX_FP_TYPE_SIZE; i++)
